#include <stdio.h>
#include <stdlib.h>
#include <getopt.h>
#include <ctype.h>
#include <sys/time.h>
#include <stdarg.h>
#include <unistd.h>
#include <fcntl.h>	
#include <errno.h>
#include <string.h>
#include <signal.h>
#include <linux/fs.h>
#include <stdint.h>
#include <sys/mman.h>
#define __USE_GNU
#include <sched.h>
#include <pthread.h>
#include <sys/resource.h>

uint64_t g_pageSize = 0;
int g_memFd = -1;
int g_time = -1;
int g_timeout = 0;
int g_continue = 0;
int g_eccCheckDuration = 60;
int g_checkECC = 0;
int g_stderr = 0;

struct testInfo
{
    int         cpu;
    uint64_t    startAddr;
    uint64_t    endAddr;
    uint32_t    errCnt;
    pthread_t   pThread;
};

struct testInfo *g_pTestInfoList = NULL;

static void logPrint(int err, const char *format, ...)
{
    char    buffer[256];
    va_list args;
    time_t  rawtime;
    struct  tm * pTimeinfo;

    va_start(args, format);
    vsprintf(buffer, format, args);
    va_end(args);

    time(&rawtime);
    pTimeinfo = localtime(&rawtime);
    printf("[%02d/%02d %02d:%02d:%02d] %s",
           pTimeinfo->tm_mon + 1, pTimeinfo->tm_mday,
           pTimeinfo->tm_hour, pTimeinfo->tm_min, pTimeinfo->tm_sec,
           buffer);

    fflush(stdout);

    if (err && g_stderr)
    {
        fprintf(stderr, "[%02d/%02d %02d:%02d:%02d] %s",
                        pTimeinfo->tm_mon + 1, pTimeinfo->tm_mday,
                        pTimeinfo->tm_hour, pTimeinfo->tm_min, pTimeinfo->tm_sec,
                        buffer);

        fflush(stderr);
    }


}

static void exitFunc()
{
    if (g_memFd > 0)
    {
        close(g_memFd);
        g_memFd = -1;
    }
    if (g_pTestInfoList)
    {
        free(g_pTestInfoList);
        g_pTestInfoList = NULL;
    }

    logPrint(0, "Exit\n");
}

static void terminate()
{
    g_continue = 0;
}

static int randomDataTest(uint64_t start, uint64_t size)
{
    uint8_t *pAddr = (uint8_t *)mmap(0, g_pageSize, (PROT_READ | PROT_WRITE), MAP_SHARED, g_memFd, start);
    uint8_t *pTempAddr;
    uint8_t pattern;
    uint64_t tempSize;

    if (pAddr == MAP_FAILED)
    {
        logPrint(1, "Error: mmap() for 0x%llx is failed\n", start);
        return 1;
    }

    pattern = rand() % 0x100;
    pTempAddr = pAddr;
    tempSize = size;
    while (tempSize--)
    {
        *pTempAddr = pattern;
        pTempAddr++;
    }

    pTempAddr = pAddr;
    tempSize = size;
    while (tempSize--)
    {
        if (*pTempAddr != pattern)
        {
            munmap((void *)pAddr, g_pageSize);
            logPrint(1, "Error: randomDataTest() is failed at 0x%llx ~ 0x%llx\n",
                    (unsigned long long)start,
                    (unsigned long long)(start +  size - 1));
            return 1;
        }
        pTempAddr++;
    }

    munmap(pAddr, g_pageSize);
    return 0;
}

static void *memTestFunc(void *param)
{
    cpu_set_t mask;
    struct testInfo *pInfo = (struct testInfo *)param;
    uint64_t address, range, offset, size;

    CPU_ZERO (&mask);
    CPU_SET (pInfo->cpu, &mask);

    pthread_setcancelstate(PTHREAD_CANCEL_ENABLE, NULL);
    pthread_setcanceltype (PTHREAD_CANCEL_DEFERRED, NULL);

    if (sched_setaffinity(getpid(), sizeof(mask), &mask) < 0)
    {
        logPrint(1, "Error: CPU%d sched_setaffinit() is failed\n", pInfo->cpu);
        exit(EXIT_FAILURE);
    }
    
    logPrint(0, "CPU%d run random address test for 0x%llx - 0x%llx\n", pInfo->cpu, pInfo->startAddr, pInfo->endAddr);

    range = pInfo->endAddr - pInfo->startAddr + 1;
    while (1)
    {
        pthread_testcancel();

        offset = (((uint64_t)rand()) << 32) | (uint64_t)rand();
        address = pInfo->startAddr + (offset % range);
        // Align test start address for page
        address &= ~(g_pageSize - 1);

        if ((address + g_pageSize) > pInfo->endAddr)
            size = pInfo->endAddr - address + 1;
        else
            size = g_pageSize;

        if (randomDataTest(address, size) != 0)
            pInfo->errCnt++;
    }

    return NULL;
}

static int clearECC(void)
{
    return system("lemur mem ecc clear &> /dev/null");
}

static int checkECC(void)
{
    FILE *pFile = popen("lemur mem ecc check", "r");
    char line[128];
    int errCount;

    if (pFile == NULL)
    {
        logPrint(1, "Error: popen \"lemur mem ecc check\" is failed\n");
        return 1;
    }

    fgets(line, sizeof(line), pFile);
    sscanf(line, "ECC error counts:%d", &errCount);

    if (errCount == 0)
    {
        pclose(pFile);
        return 0;
    }

    logPrint(1, "%s\n", line);
    while((fgets(line, sizeof(line), pFile)) != NULL)
        logPrint(1, "%s\n", line);

    pclose(pFile);

    return errCount;
}

static void alarmHandler (int signum)
{
    int eccCheckDuration = g_eccCheckDuration;

    // ECC check is disabled
    if (eccCheckDuration == 0)
    {
        g_time = 0;
        g_timeout = 1;
        return;
    }

    // Meet the last duration time
    if ((0 <= g_time) && (g_time <= eccCheckDuration))
    {
        g_checkECC = 1;
        g_time = 0;
        g_timeout = 1;
        return;
    }

    if (g_time > eccCheckDuration)
    {
        g_time -= eccCheckDuration;
        if (g_time < eccCheckDuration)
            eccCheckDuration = g_time;
    }

    g_checkECC = 1;
    alarm(eccCheckDuration);
}

static int help(void)
{
	printf("Usage: memstress start end --c --e 30 --t 60 --s\n"
	       "\t--help\t\tDisplay this information.\n"
	       "\t--continue\tDo not stop on error.\n"
	       "\t--eccduration\t Duration time for ECC check (default: 60 seconds).\n"
	       "\t--timeout N\tTimeout after N seconds.\n"
	       "\t--stderr\tOutput errors to stderr.\n");

	return 0;
}

int main (int argc, char **argv)
{
    int ret = 0;
    int stopOnError = 1;
    struct sigaction signalAction;
    int cpuNum = 0, i;
    uint64_t startAddr = 0xFFFFFFFFFFFFFFFF, endAddr = 0xFFFFFFFFFFFFFFFF, blockSize;
    int totalErrCnt = 0;


	const struct option longopts[] = {
		{"help", no_argument, NULL, 'h'},
        {"continue", no_argument, NULL, 'c'},
        {"eccduration", required_argument, NULL, 'e'},
		{"timeout", required_argument, NULL, 't'},
        {"stderr", no_argument, NULL, 's'},
		{0, 0, 0, 0}
	};


	if (argc < 3)
		return help();

    argv++;
    if (isdigit((*argv)[0]))
        startAddr = strtoull(*argv, NULL, 0);
    else
        return help();
    argv++;
    if (isdigit((*argv)[0]))
        endAddr = strtoull(*argv, NULL, 0);
    else
        return help();

    g_pageSize = (uint64_t)sysconf(_SC_PAGE_SIZE);

    // Align start address for page
    if (startAddr % g_pageSize)
    {
        startAddr &= ~(g_pageSize - 1);
        startAddr += g_pageSize;
    }

    if (startAddr >= endAddr)
        return help();

    argc -= 2;

	while (1)
	{
		ret = getopt_long (argc, argv, "hce:t:s", longopts, NULL);
		if (ret < 0)
			break;

		switch (ret)
		{
            case 'h':
                return help();
                break;
            case 'c':
                stopOnError = 0;
                break;
            case 'e':
                g_eccCheckDuration = atoi(optarg);
                if (g_eccCheckDuration < 0)
                    return help();
                break;
            case 't':
                g_time = atoi(optarg);
                if (g_time <= 0)
                    g_time = -1;
                break;
            case 's':
                g_stderr = 1;
                break;
            default:
                return (-EINVAL);
		}
	}

	if (clearECC())
	{
	    logPrint(1, "Error: Clear ECC is failed\n");
	    exit (EXIT_FAILURE);
	}
	else
	{
	    logPrint(0, "Clear ECC\n");
	}

	atexit(exitFunc);

	memset(&signalAction, 0, sizeof(struct sigaction));
	signalAction.sa_handler = terminate;
	sigaction(SIGTERM, &signalAction, NULL);

	cpuNum = sysconf(_SC_NPROCESSORS_ONLN);

	g_pTestInfoList = (struct testInfo *)malloc(sizeof(struct testInfo) * cpuNum);

    if ((g_memFd = open("/dev/mem", O_RDWR | O_SYNC)) == -1)
    {
        logPrint(1, "Error: open /dev/mem is failed: %s\n", strerror(errno));
        exit (EXIT_FAILURE);
    }

    if (g_time != -1)
    {
        logPrint(0, "Run memstress within 0x%llx - 0x%llx for %d seconds\n", startAddr, endAddr, g_time);

        signal(SIGALRM, alarmHandler);
        if (g_eccCheckDuration && (g_time > g_eccCheckDuration))
            alarm(g_eccCheckDuration);
        else
            alarm(g_time);
    }
    else
    {
        logPrint(0, "Run memstress within 0x%llx - 0x%llx\n", startAddr, endAddr);

        if (g_eccCheckDuration)
        {
            signal(SIGALRM, alarmHandler);
            alarm(g_eccCheckDuration);
        }
    }

    if (g_eccCheckDuration)
        logPrint(0, "Check ECC for every %d seconds\n", g_eccCheckDuration);

	// Divide test range into several blocks based on threads
	blockSize = (endAddr - startAddr + 1) / cpuNum;
    while ((cpuNum > 1) && (blockSize < g_pageSize))
        blockSize = (endAddr - startAddr + 1) / --cpuNum;

	// Align test block for page
	if (blockSize % g_pageSize)
	    blockSize &= ~(g_pageSize -1);

	for (i = 0; i < cpuNum; i++)
	{
	    g_pTestInfoList[i].cpu = i;
	    g_pTestInfoList[i].startAddr = startAddr;
        if (i != (cpuNum - 1))
            g_pTestInfoList[i].endAddr = startAddr + blockSize - 1;
        else
            g_pTestInfoList[i].endAddr = endAddr;
        startAddr += blockSize;
        g_pTestInfoList[i].errCnt = 0;

        if (pthread_create(&g_pTestInfoList[i].pThread, NULL, memTestFunc, (void *)&g_pTestInfoList[i]) != 0)
        {
            logPrint(1, "Error: thread creation is failed\n");

            for (i--; i >= 0; i--)
            {
                pthread_cancel(g_pTestInfoList[i].pThread);
            }
            exit (EXIT_FAILURE);
        }
	}

	g_continue = 1;

    while (g_continue)
    {
        usleep(100);

        // Stop if error occurred
        if (stopOnError)
        {
            for (i = 0; i < cpuNum; i++)
                totalErrCnt += g_pTestInfoList[i].errCnt;

            if (totalErrCnt)
                break;
        }

        if (g_checkECC)
        {
            g_checkECC = 0;
            totalErrCnt = checkECC();
            if (totalErrCnt && stopOnError)
                break;
        }

        // Time is up
        if (g_timeout)
        {
            logPrint(0, "Time is up\n");
            break;
        }
    }

    for (i = 0; i < cpuNum; i++)
        pthread_cancel(g_pTestInfoList[i].pThread);

	return totalErrCnt;
}

