/*******************************************************************************

NAME            fpgaCwtDiag.c
SUMMARY         Zebulon fpga CWT function diagnostics
VERSION         %version: 1 %
UPDATE DATE     %date_modified: Apr 28 15:45 2009 %
PROGRAMMER      %created_by:    Jim Tu %

        Copyright 2009 Quanta Corporation. All Rights Reserved.

DESCRIPTION:
        This file has code to test CWT function of Zebulon fpga

NOTES:

REFERENCE:

*******************************************************************************/

#include <linux/kernel.h>
#include <linux/delay.h>

#include <commTypes.h>
#include <commRpt.h>
#include <commMEC.h>

#include "fpgaRegLib.h"
#include "fpgaCwtDiag.h"
#include "fpgaIntMaskDiag.h"
#include "fpgaCmdLib.h"
#include "fpgaNvsram.h"

/***  Inport Function  ***/
EXTERN UINT8   	ctrlBootCountGet(VOID);
EXTERN UINT8   	ctrlBootCountCompareGet(VOID);
EXTERN VOID 	ctrlBootCountCompareSet(VOID);

EXTERN VOID 	fpgaNvsramUnlockAll(VOID);
//EXTERN VOID 	fpgaClearDirectedReset(VOID);

/****************************/
/**  CWT Control Function  **/
/****************************/

/*******************************************************************************
* PROCEDURE
*
* NAME:         fpgaCwtProtectSet
*
* SUMMARY:      Enable/disable CWT protect
*
* SCOPE:        private
*
* DESCRIPTION:  Enable/disable CWT protect
*
* RESTRICTIONS:
* 
* RETURNS:      
*/
VOID fpgaCwtProtectSet
 (
 BOOLEAN status
 )
{
    INT32 result;
    
    if(status)
	{	//enable CWT protect
        result = fpgaRegisterWrite(FPGA_CWT_PROTECT, 1);
    }
	else
	{	//disable CWT protect
        result = fpgaRegisterWrite(FPGA_CWT_PROTECT, 0);
    }
}

/*******************************************************************************
* PROCEDURE
*
* NAME:         fpgaCwtEnableSet
*
* SUMMARY:      Enable/disable CWT
*
* SCOPE:        private
*
* DESCRIPTION:  Enable/disable CWT
*
* RESTRICTIONS:
* 
* RETURNS:      
*/
VOID fpgaCwtEnableSet
 (
 BOOLEAN status
 )
{
    INT32 result;
    
    if(status)
	{	//enable CWT
		fpgaCwtProtectSet(FALSE);
        result = fpgaRegisterWrite(FPGA_CWT_ENABLE, 1);
		fpgaCwtProtectSet(TRUE);
    }
	else
	{	//disable CWT
		fpgaCwtProtectSet(FALSE);
        result = fpgaRegisterWrite(FPGA_CWT_ENABLE, 0);
		fpgaCwtProtectSet(TRUE);
    }
}

/*******************************************************************************
* PROCEDURE
*
* NAME:         fpgaCwtTrigger
*
* SUMMARY:      Trigger CWT
*
* SCOPE:        private
*
* DESCRIPTION:  Read CWT key from CWT key register and write it INT32o 
*				the control register to trigger CWT. If the re-trigger 
*				period is longer than 400ms, the CWT fault INT32errupt will 
*				be asserted. If the re-trigger period is longer than 800ms,
*				the controller will be reset.
*
* RESTRICTIONS:
* 
* RETURNS:      
*/
VOID fpgaCwtTrigger(VOID)
{
    fpgaRegisterWrite(FPGA_CWT_CONTROL, fpgaRegisterRead(FPGA_CWT_KEY));
}

/*******************************************************************************
* PROCEDURE
*
* NAME:         fpgaCwtFault
*
* SUMMARY:      verifid if CWT is in Faulted state
*
* SCOPE:        private
*
* DESCRIPTION:  verifid if CWT is in Faulted state by reading CWT_FAULTn bit in
*				Miscellaneous Information register. The bit is valid after a reset.
*
* RESTRICTIONS:
* 
* RETURNS:      TRUE:	Faulted State
*				FALSE:	Normal
*/
INT32 fpgaCwtFault(VOID)   //true: CWT in Faulted state
{
    if(fpgaRegisterRead(FPGA_LEGACY_MISCELLANEOUS_INFORMATION_1) & CWT_FAULT_BIT)
	{
		//1: the CWT is not in Faulted state
        return FALSE;
    }
	else
	{
		//0: the CWT is in Faulted state
        return TRUE;
    }
}

/*******************************************************************************
* PROCEDURE
*
* NAME:         fpgaCwtNmi
*
* SUMMARY:      verifid if CWT is in Warning state
*
* SCOPE:        private
*
* DESCRIPTION:  verifid if CWT is in Warning state by reading WDT_NMI_INT bit in
*				High Priority Interrupt Source register.
*
* RESTRICTIONS:
* 
* RETURNS:      TRUE:	Warning State
*				FALSE:	Normal
*/
INT32 fpgaCwtNmi(VOID)
{
    if(fpgaRegisterRead(FPGA_INTERRUPT_SOURCE_INT_B) & WDT_INT_BIT)
	{	//1: the CWT NMI is asserted
        return TRUE;
    }
	else
	{	//0: the CWT NMI is not asserted
        return FALSE;
    }
}

/********************************/
/**  CWT Diagnostics Function  **/
/********************************/
/*******************************************************************************
* PROCEDURE
*
* NAME:         fpgaCwtWarningDiag
*
* SUMMARY:      diagnostics CWT Warning state mechanism
*
* SCOPE:        private
*
* DESCRIPTION:  Not re-trigger the CWT over 400ms and verify if the CWT 
*				enters the Warning state and send the NMI to CPU
*
* RESTRICTIONS:
* 
* RETURNS:      TRUE:	Pass
*				FALSE:	Fail
*/
INT32 fpgaCwtWarningDiag
 (
 VOID* 	diagSeqRecordCwtP,		//test checkpoint
 VOID* 	diagResultRecordCwtP	//test result flag
 )
{
    INT32 result = TRUE;
	INT32 i;
    
	//set controller boot count
    ctrlBootCountCompareSet();
    
	//start and trigger CWT
    fpgaCwtEnableSet(TRUE);   
    fpgaCwtTrigger();
	
    if(fpgaCwtNmi())
	{   //if the NMI has been asserted, it means the CWT is not functional
		result = FALSE;
        *(volatile UINT8*)diagResultRecordCwtP |= FPGA_CWT_RESULT_WARNING_FAIL;
		printk("[FPGA CWT] CWT NMI is abnormally asserted.\n");
    }
    
    if(fpgaCwtFault())
	{	//verify CWT_FAULTn, at this time it should not be asserted
        result = FALSE;
		*(volatile UINT8*)diagResultRecordCwtP |= FPGA_CWT_RESULT_FAULTED_FAIL;
		printk("[FPGA CWT] CWT CWT_FAULTn is abnormally asserted.\n");
    }
	
    for(i = 0; i < FPGA_CWT_WRANING_TEST_TIMES; i++)
	{
	//retrigger CWT
    fpgaCwtTrigger();
	mdelay(FPGA_CWT_WRANING_DELAY);             
	//after FPGA_CWT_WRANING_DELAY, the CWT should be in the Warning State
    
	if(!fpgaCwtNmi())
	{	//not in Warning state, the CWT is not functional
        //turn off CWT   
        fpgaCwtTrigger();
		*(volatile UINT8*)diagResultRecordCwtP |= FPGA_CWT_RESULT_WARNING_FAIL;
        result = FALSE;
		printk("[FPGA CWT] CWT NMI is not functional after CWT trigger %dms.\n", FPGA_CWT_WRANING_DELAY);
    }
	else
	{	//in Warning state, the CWT is functional
        //turn off CWT
		fpgaCwtTrigger();
    }
    
	fpgaCwtTrigger();
	mdelay(100);
	if(fpgaCwtNmi())
	{   //if the NMI has been asserted, it means the CWT is not functional
		result = FALSE;
        *(volatile UINT8*)diagResultRecordCwtP |= FPGA_CWT_RESULT_WARNING_FAIL;
		printk("[FPGA CWT] CWT NMI is abnormally asserted.\n");
    }
	
    if(fpgaCwtFault())
	{	//verify CWT_FAULTn, at this time it should not be asserted
        *(volatile UINT8*)diagResultRecordCwtP |= FPGA_CWT_RESULT_FAULTED_FAIL;
        result = FALSE;
		printk("[FPGA CWT] CWT CWT_FAULTn is abnormally asserted.\n");
    }
	fpgaCwtTrigger();
    }
	
	fpgaCwtEnableSet(FALSE);
	//set to next checkpoint
    *(volatile UINT8*)diagSeqRecordCwtP = FPGA_CWT_SEQ_WARNING_TIME_CORRECT;
    printk("[FPGA CWT] CWT Warning state test is done.\n");
    
    return result;
}

/*******************************************************************************
* PROCEDURE
*
* NAME:         fpgaCwtFaultedDiag
*
* SUMMARY:      diagnostics CWT Faulted state mechanism
*
* SCOPE:        private
*
* DESCRIPTION:  Not re-trigger the CWT over 800ms and verify if the CWT 
*				enters the Faulted state and reset the controller
*
* RESTRICTIONS:
* 
* RETURNS:      TRUE:	Pass
*				FALSE:	Fail
*/
INT32 fpgaCwtFaultedDiag
 (
 VOID* 	diagSeqRecordCwtP,		//test checkpoint
 VOID* 	diagResultRecordCwtP,	//test result flag
 INT32 	mode, 					//diagnostics mode in CWT Faulted state test
 UINT8 	bootingC,				//controller boot count value 
 UINT8 	bootingCC				//controller boot count compare value
 )
{
    INT32 result = TRUE;
    
    if(mode == 0)	/***   just start CWT faulted state diagnostics   ***/
	{   
		//set controller boot count compare byte
        ctrlBootCountCompareSet();
	
		//start CWT
        fpgaCwtEnableSet(TRUE);       
        fpgaCwtTrigger();
	
        mdelay(FPGA_CWT_FAULTED_DELAY);                
                    
        //after 800ms, the controller should be already rebooted (bootingC++)
        
		printk("[FPGA CWT] CWT Reset is not functional.\n");                            
        if(!fpgaCwtFault())
		{	//not be rebooted. CWT Fault State fail
            fpgaCwtTrigger();
			//set flag
            *(volatile UINT8*)diagResultRecordCwtP |= FPGA_CWT_RESULT_FAULTED_FAIL;
            result = FALSE;
			printk("[FPGA CWT] CWT CWT_FAULTn is abnormally asserted after trigger.\n");
        }         
    }
	else if(mode == 1)    /***   verify controller is rebooted   ***/
	{  
        if(bootingCC == bootingC)
		{	//the controller has not been reset. CWT Fault State fail
            *(volatile UINT8*)diagResultRecordCwtP |= FPGA_CWT_RESULT_RESET_FAIL;
            result = FALSE;
			printk("[FPGA CWT] Boot count does not increase. CWT Reset is not functional.\n");
        }
        
        if(!fpgaCwtFault())
		{   //Faulted state should be latched
            fpgaCwtTrigger();
            *(volatile UINT8*)diagResultRecordCwtP |= FPGA_CWT_RESULT_FAULTED_FAIL;
            result = FALSE;
			printk("[FPGA CWT] CWT fault latch is failed.\n");
        }
    }

	//turn off CWT
    fpgaCwtEnableSet(FALSE);   
    
    return result;
}

/*********************************/
/**  Main Diagnostics Function  **/
/*********************************/
/*******************************************************************************
* PROCEDURE
*
* NAME:         fpgaCwtDiag
*
* SUMMARY:      diagnostics CWT mechanism
*
* SCOPE:        public
*
* DESCRIPTION:  diagnostics CWT Warning and Faulted state mechanism
*
* RESTRICTIONS:
* 
* RETURNS:      TRUE:	Pass
*				FALSE:	Fail
*/
INT32 fpgaCwtDiag(VOID)
{
    INT32 	resultWarning = TRUE;
    INT32 	resultFaulted = TRUE;
    
    //the following values are stored in NVSRAM
    VOID* 	diagSeqRecordCwtP;		//test checkpoint
	VOID* 	diagResultRecordCwtP;	//test result flag
    UINT8 	bootingCount, bootingCountComparison;
    
    fpgaNvsramUnlockAll();
    
    //load bootingCount and bootingCount_compare
    bootingCount = ctrlBootCountGet();
    bootingCountComparison = ctrlBootCountCompareGet();

    diagSeqRecordCwtP = gFpgaBar2VirtualP + NVSRAM_ADDRESS_BASE + FPGA_CWT_DIAG_SEQ_RECORD_OFFSET;
	diagResultRecordCwtP = gFpgaBar2VirtualP + NVSRAM_ADDRESS_BASE + FPGA_CWT_RESULT_OFFSET;
    
    //verify if this the first time in CWT test
    if( (*(volatile UINT8*)(gFpgaBar2VirtualP + NVSRAM_ADDRESS_BASE + FPGA_CWT_ENCODED_UINT8_0_LOCATION) != FPGA_CWT_ENCODED_PATTERN_0) |    
        (*(volatile UINT8*)(gFpgaBar2VirtualP + NVSRAM_ADDRESS_BASE + FPGA_CWT_ENCODED_UINT8_1_LOCATION) != FPGA_CWT_ENCODED_PATTERN_1) |    
        (*(volatile UINT8*)(gFpgaBar2VirtualP + NVSRAM_ADDRESS_BASE + FPGA_CWT_ENCODED_UINT8_2_LOCATION) != FPGA_CWT_ENCODED_PATTERN_2) )
	{
		//set flag & checkpoINT32
    	*(volatile UINT8*)diagSeqRecordCwtP = FPGA_CWT_SEQ_NORMAL;
		*(volatile UINT8*)diagResultRecordCwtP = FPGA_CWT_RESULT_GOOD;
	
		//set syndrome
		*(volatile UINT8*)(gFpgaBar2VirtualP + NVSRAM_ADDRESS_BASE + FPGA_CWT_ENCODED_UINT8_0_LOCATION) = FPGA_CWT_ENCODED_PATTERN_0;    
        *(volatile UINT8*)(gFpgaBar2VirtualP + NVSRAM_ADDRESS_BASE + FPGA_CWT_ENCODED_UINT8_1_LOCATION) = FPGA_CWT_ENCODED_PATTERN_1;    
        *(volatile UINT8*)(gFpgaBar2VirtualP + NVSRAM_ADDRESS_BASE + FPGA_CWT_ENCODED_UINT8_2_LOCATION) = FPGA_CWT_ENCODED_PATTERN_2;	
    }
    
    printk("[FPGA CWT] CWT test start.\n");
    //fpgaClearDirectedReset();
	
    //just start CWT diagnostics
    if(*(volatile UINT8*)diagSeqRecordCwtP == FPGA_CWT_SEQ_NORMAL)
	{   
		//set checkpoint
        *(volatile UINT8*)diagSeqRecordCwtP = FPGA_CWT_SEQ_WARNING;
		printk("[FPGA CWT] CWT Warning state test start.\n");
        
		//enter CWT Warning state diag.
		resultWarning = fpgaCwtWarningDiag(diagSeqRecordCwtP, diagResultRecordCwtP);
		*(volatile UINT8*)diagSeqRecordCwtP = FPGA_CWT_SEQ_WARNING_TIME_CORRECT;
		//printk("[FPGA CWT] diagSeqRecordCwtP: %lld\n", (UINT64)diagSeqRecordCwtP);    
    }
	
    //Warning state daig. is done and CWT timer is not correct
    if(*(volatile UINT8*)diagSeqRecordCwtP == FPGA_CWT_SEQ_WARNING)
	{   
		//set checkpoint
        *(volatile UINT8*)diagSeqRecordCwtP = FPGA_CWT_SEQ_FAULTED;
        printk("[FPGA CWT] CWT Timer is not correct!!\n");
        
		//set result
		*(volatile UINT8*)diagResultRecordCwtP |= FPGA_CWT_RESULT_TIMER_FAIL;
        resultWarning = FALSE;
		printk("[FPGA CWT] CWT Faulted state test start.\n");
	
        //enter CWT Faulted state diag.
        resultFaulted = fpgaCwtFaultedDiag(diagSeqRecordCwtP, diagResultRecordCwtP, 
											0, bootingCount, bootingCountComparison);
    }
    
    //Warning state daig. is done and CWT timer is correct
    if(*(volatile UINT8*)diagSeqRecordCwtP == FPGA_CWT_SEQ_WARNING_TIME_CORRECT)
	{   
		//set checkpoint
        *(volatile UINT8*)diagSeqRecordCwtP = FPGA_CWT_SEQ_FAULTED;
        printk("[FPGA CWT] CWT Faulted state test start.\n");
	
        //enter CWT Faulted state diag.
        resultFaulted = fpgaCwtFaultedDiag(diagSeqRecordCwtP, diagResultRecordCwtP, 
											0, bootingCount, bootingCountComparison);
    }
    /***  should be rebooted  ***/
    //after reboot
    if(*(volatile UINT8*)diagSeqRecordCwtP == FPGA_CWT_SEQ_FAULTED)
	{     
        //check reboot
        resultFaulted = fpgaCwtFaultedDiag(diagSeqRecordCwtP, diagResultRecordCwtP, 
											1, bootingCount, bootingCountComparison);
        *(volatile UINT8*)diagSeqRecordCwtP = FPGA_CWT_SEQ_NORMAL;
		printk("[FPGA CWT] CWT test is done\n");
    	printk("[FPGA CWT] ctrlBootCountGet: 0x%02X\n", ctrlBootCountGet());
		printk("[FPGA CWT] ctrlBootCountCompareGet: 0x%02X\n", ctrlBootCountCompareGet());
    }
	*(volatile UINT8*)diagSeqRecordCwtP = FPGA_CWT_SEQ_NORMAL;
	
	/**********************/
    /***  print error  ***/
	/**********************/
	//print CWT Warning state fail
    if((*(volatile UINT8*)diagResultRecordCwtP & FPGA_CWT_RESULT_WARNING_FAIL) == 1)
	{
    	commReportError(MEC_FPGA, FPGA_CWT_MEC, FPGA_CWT_WARNING_ERROR, MSG_FPGA_CWT_WARNING_ERROR, 0, 0, 0, 0);
		resultWarning = FALSE;
    }
    
	//print CWT Faulted state fail
    if((*(volatile UINT8*)diagResultRecordCwtP & FPGA_CWT_RESULT_FAULTED_FAIL) == 1)
	{
    	commReportError(MEC_FPGA, FPGA_CWT_MEC, FPGA_CWT_FAULTED_ERROR, MSG_FPGA_CWT_FAULTED_ERROR, 0, 0, 0, 0);
		resultFaulted = FALSE;
    }
    
	//print CWT reset fail
    if((*(volatile UINT8*)diagResultRecordCwtP & FPGA_CWT_RESULT_RESET_FAIL) == 1)
	{
    	commReportError(MEC_FPGA, FPGA_CWT_MEC, FPGA_CWT_RESET_ERROR, MSG_FPGA_CWT_RESET_ERROR, 0, 0, 0, 0);
		resultFaulted = FALSE;
    }
    
	//print CWT timer fail
    if((*(volatile UINT8*)diagResultRecordCwtP & FPGA_CWT_RESULT_TIMER_FAIL) == 1)
	{
    	commReportError(MEC_FPGA, FPGA_CWT_MEC, FPGA_CWT_TIMER_ERROR, MSG_FPGA_CWT_TIMER_ERROR, 0, 0, 0, 0);
		resultWarning = FALSE;
    }
    
    //reset syndrome
    *(volatile UINT8*)(gFpgaBar2VirtualP + NVSRAM_ADDRESS_BASE + FPGA_CWT_ENCODED_UINT8_0_LOCATION) = 0;    
    *(volatile UINT8*)(gFpgaBar2VirtualP + NVSRAM_ADDRESS_BASE + FPGA_CWT_ENCODED_UINT8_1_LOCATION) = 0;    
    *(volatile UINT8*)(gFpgaBar2VirtualP + NVSRAM_ADDRESS_BASE + FPGA_CWT_ENCODED_UINT8_2_LOCATION) = 0;
    
    return (resultWarning && resultFaulted);
}


VOID fpgaCwtService(UINT32 delay, UINT32 times)
{
	UINT32 i = 0;
	printk("[FPGA CWT] CWT is enabled. Service every %d ms.\n", delay);
	
     
	
	if(fpgaCwtNmi())
	{	//not in Warning state, the CWT is not functional
		printk("[FPGA CWT] CWT NMI is inserted after CWT Enable! Fail.\n");
    }
	    
    
	
	do
	{
		fpgaCwtEnableSet(TRUE);
		fpgaCwtTrigger();
		if(fpgaCwtNmi())
		{	//not in Warning state, the CWT is not functional
			printk("[FPGA CWT] CWT NMI is inserted\n");
    	}
		
		fpgaCwtTrigger();
		mdelay(delay);
		
		if(!fpgaCwtNmi())
		{	//not in Warning state, the CWT is not functional
			printk("[FPGA CWT] CWT NMI is not inserted\n");
    	}
		
		fpgaCwtTrigger();
		//turn off CWT
    	fpgaCwtEnableSet(FALSE);
		printk("[FPGA CWT] Next Round.\n");
		i++;
	}while(i < times);
	
		   
	printk("[FPGA CWT] CWT is disabled\n");
}


