/*******************************************************************************

NAME            fpgaNvsramLockDiag.c
SUMMARY         Zebulon FPGA NVSRAM lock function diagnostics
VERSION         %version: 1 %
UPDATE DATE     %date_modified: Apr 27 18:20 2009 %
PROGRAMMER      %created_by:    Jim Tu %

        Copyright 2009 Quanta Corporation. All Rights Reserved.

DESCRIPTION:
        This file has code to test the NVSRAM lock function of Zebulon FPGA

NOTES:

REFERENCE:

*******************************************************************************/
#include <linux/module.h>

#include <linux/kernel.h>
#include <linux/vmalloc.h>
#include <linux/io.h>
#include <linux/slab.h>
#include <linux/delay.h>

#include <commTypes.h>
#include <commRpt.h>
#include <commMEC.h>

#include "fpgaNvsram.h"
#include "fpgaRegLib.h"
#include "fpgaCmdLib.h"


/*******************************************************************************
* PROCEDURE
*
* NAME:         fpgaNvsramLockDiag
*
* SUMMARY:      Test NVSRAM lock function of Zebulom FPGA
*
* SCOPE:        public
*
* DESCRIPTION:  1.Lock entire NVSRAM and write test pattern to NVSRAM.
*				2.Verify if the NVSRAM content has been modified.
*
* RESTRICTIONS:
* 
* RETURNS:      
*/
INT32 fpgaNvsramLockDiag(VOID)
{   
	INT32	result = TRUE;
	INT32	blockCount, byteCount;
	VOID*	tempBackupBaseP;
    
	//allocate memory for temperal NVSRAM backup
	tempBackupBaseP = kmalloc(NVSRAM_TOTAL_SIZE, GFP_KERNEL);
	if(tempBackupBaseP == NULL)
	{
    	printk("[FPGA NVSRAM] Cannot allocate memory for NVSRAM backup!!");
		return FALSE;
	}
    
	//backup NVSRAM content
    memcpy(tempBackupBaseP, gFpgaBar2VirtualP + NVSRAM_ADDRESS_BASE, NVSRAM_TOTAL_SIZE);
    
	fpgaRegisterShow(FPGA_NVSRAM_LOCK);
	
	printk("[FPGA NVSRAM] Start Lock Test...\n");

	for(blockCount = 0; blockCount < FPGA_NVSRAM_LOCK_ENTIRE; blockCount++)
	{
		printk("\n");
		fpgaRegisterShow(FPGA_NVSRAM_LOCK);
		
        for(byteCount = 0; byteCount < NVSRAM_BLOCK_SIZE; byteCount++)
		{                        
			//unlock entire NVSRAM
    		fpgaNvsramUnlockAll();
			
			*(volatile UINT8*)(gFpgaBar2VirtualP + NVSRAM_ADDRESS_BASE + (blockCount * NVSRAM_BLOCK_SIZE) + byteCount) = 0x55; 
			//verify
			ndelay(200);
			if((*(volatile UINT8*)(gFpgaBar2VirtualP + NVSRAM_ADDRESS_BASE + (blockCount * NVSRAM_BLOCK_SIZE) + byteCount)) != 0x55)
			{
				commReportError(MEC_FPGA, FPGA_NVSRAM_LOCK_MEC, (UINT16)(FPGA_NVSRAM_LOCK_ERROR_BASE + blockCount),
									MSG_FPGA_NVSRAM_LOCK_ERROR, blockCount, byteCount, 1, DG_FPGA_NVSRAM_FORMAT);
				printk("0x%02X\n", *(volatile UINT8*)(gFpgaBar2VirtualP + NVSRAM_ADDRESS_BASE + (blockCount * NVSRAM_BLOCK_SIZE) + byteCount));
				result = FALSE;
			}
			
			//lock entire NVSRAM
			fpgaNvsramLockAll();

			//write pattern 0
			*(volatile UINT8*)(gFpgaBar2VirtualP + NVSRAM_ADDRESS_BASE + 
							(blockCount * NVSRAM_BLOCK_SIZE) + byteCount) = NVSRAM_LOCK_PATTERN_0;
			ndelay(200);
			//verify
			if((*(volatile UINT8*)(gFpgaBar2VirtualP + NVSRAM_ADDRESS_BASE + 
								(blockCount * NVSRAM_BLOCK_SIZE) + byteCount)) == NVSRAM_LOCK_PATTERN_0)
			{    
				commReportError(MEC_FPGA, FPGA_NVSRAM_LOCK_MEC, (UINT16)(FPGA_NVSRAM_LOCK_ERROR_BASE + blockCount),
								MSG_FPGA_NVSRAM_LOCK_ERROR, blockCount, byteCount, 1, DG_FPGA_NVSRAM_FORMAT);
				fpgaRegisterShow(FPGA_NVSRAM_LOCK);
				printk("0x%02X\n", *(volatile UINT8*)(gFpgaBar2VirtualP + NVSRAM_ADDRESS_BASE + (blockCount * NVSRAM_BLOCK_SIZE) + byteCount));
				result = FALSE;
			}
			
			//lock entire NVSRAM
			fpgaNvsramLockAll();
			
			//write pattern 1
			*(volatile UINT8*)(gFpgaBar2VirtualP + NVSRAM_ADDRESS_BASE + 
							(blockCount * NVSRAM_BLOCK_SIZE) + byteCount) = NVSRAM_LOCK_PATTERN_1;  
			ndelay(200);
			//verify
			if((*(volatile UINT8*)(gFpgaBar2VirtualP + NVSRAM_ADDRESS_BASE + 
								(blockCount * NVSRAM_BLOCK_SIZE) + byteCount)) == NVSRAM_LOCK_PATTERN_1)
			{            
				commReportError(MEC_FPGA, FPGA_NVSRAM_LOCK_MEC, (UINT16)(FPGA_NVSRAM_LOCK_ERROR_BASE + blockCount),
                                MSG_FPGA_NVSRAM_LOCK_ERROR, blockCount, byteCount, 2, DG_FPGA_NVSRAM_FORMAT);
				fpgaRegisterShow(FPGA_NVSRAM_LOCK);
				printk("0x%02X\n", *(volatile UINT8*)(gFpgaBar2VirtualP + NVSRAM_ADDRESS_BASE + (blockCount * NVSRAM_BLOCK_SIZE) + byteCount));
				result = FALSE;
			}
        }
        
        commPostProgress("[FPGA NVSRAM] Lock Test Progress", blockCount + 1, FPGA_NVSRAM_LOCK_ENTIRE);
    }
	    
	//restore NVSRAM content
    fpgaNvsramUnlockAll();
    memcpy(gFpgaBar2VirtualP + NVSRAM_ADDRESS_BASE, tempBackupBaseP, NVSRAM_TOTAL_SIZE);
    
    //free memory
    kfree(tempBackupBaseP);
    
    return result;
}



VOID fpgaNvsramShow
 (
 UINT32	fpgaOffset	//register offset from BAR2
 )
{
	printk("data = 0x%02X \n ",*(volatile UINT8*)(gFpgaBar2VirtualP + fpgaOffset));
}

VOID fpgaNvsramWrite
 (
 UINT32	fpgaOffset,	//register offset from BAR2
 UINT8	data
 )
{
	fpgaNvsramUnlockAll();
	*(volatile UINT8*)(gFpgaBar2VirtualP + fpgaOffset) = data;
}
