/*******************************************************************************

NAME            fpgaProgramDiag.c
SUMMARY         Zebulon fpga programming function diagnostics
VERSION         %version: 1 %
UPDATE DATE     %date_modified: Apr 28 17:00 2009 %
PROGRAMMER      %created_by:    Jim Tu %

        Copyright 2009 Quanta Corporation. All Rights Reserved.

DESCRIPTION:
        This file has code to test programming function of Zebulon fpga

NOTES:

REFERENCE:

*******************************************************************************/
#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/delay.h>

#include <commTypes.h>
#include <commRpt.h>
#include <commMEC.h>

#include "fpgaRegLib.h"
#include "fpgaProgramDiag.h"

//global flag
INT32 gEpcsDead = 0;

INT32 fpgaProgramEpcsPresent(VOID);
INT32 fpgaProgramWriteInProgress(VOID);
/*****************************************************/
/**  Program Interface <FPGA Side> Status Function  **/
/*****************************************************/

/*******************************************************************************
* PROCEDURE
*
* NAME:         fpgaReconfig
*
* SUMMARY:      Trigger FPGA programming
*
* SCOPE:        private
*
* DESCRIPTION:  Trigger FPGA programming after the new FPGA code is written into 
*				EPCS
*
* RESTRICTIONS:
* 
* RETURNS:      
*/
VOID fpgaReconfig
 (
 BOOLEAN status
 )
{
    INT32 	result;
    UINT8 	temp;
    
	temp = fpgaRegisterRead(FPGA_PROTECTED_WRITE_2);
    if(status)
	{
        temp |= FPGA_RECONFIG_BIT;
    }
	else
	{
        temp &= (~FPGA_RECONFIG_BIT);
    }
	
	fpgaEncodedRegisterWrite();
    result = fpgaRegisterWrite(FPGA_PROTECTED_WRITE_2, temp);
}


/*******************************************************************************
* PROCEDURE
*
* NAME:         fpgaProgramRoe
*
* SUMMARY:      return receive overrun error
*
* SCOPE:        private
*
* DESCRIPTION:  return receive overrun error
*
* RESTRICTIONS:
* 
* RETURNS:      TRUE:	error
*				FALSE:	good
*/
INT32 fpgaProgramRoe(VOID)
{
    if(fpgaRegisterRead(FPGA_FPGA_PROGRAM_STATUS) & ROE_BIT)
	{
        return TRUE;
    }
	else
	{
        return FALSE;
    }
}


/*******************************************************************************
* PROCEDURE
*
* NAME:         fpgaProgramToe
*
* SUMMARY:      return transmitter overrun error
*
* SCOPE:        private
*
* DESCRIPTION:  return transmitter overrun error
*
* RESTRICTIONS:
* 
* RETURNS:      TRUE:	error
*				FALSE:	good
*/
INT32 fpgaProgramToe(VOID)
{
    if(fpgaRegisterRead(FPGA_FPGA_PROGRAM_STATUS) & TOE_BIT)
	{
        return TRUE;
    }
	else
	{
        return FALSE;
    }
}

/*******************************************************************************
* PROCEDURE
*
* NAME:         fpgaProgramTmt
*
* SUMMARY:      transmitter shift-register empty
*
* SCOPE:        private
*
* DESCRIPTION:  transmitter shift-register empty
*
* RESTRICTIONS:
* 
* RETURNS:      TRUE:	empty
*				FALSE:	transaction in progress
*/
INT32 fpgaProgramTmt(VOID)
{
    if(fpgaRegisterRead(FPGA_FPGA_PROGRAM_STATUS) & TMT_BIT)
	{
        return TRUE;
    }
	else
	{
        return FALSE;
    }
}

/*******************************************************************************
* PROCEDURE
*
* NAME:         fpgaProgramTrdy
*
* SUMMARY:      return if the transmit is ready
*
* SCOPE:        private
*
* DESCRIPTION:  return if the transmit is ready
*
* RESTRICTIONS:
* 
* RETURNS:      TRUE:	TxData is empty
*				FALSE:	TxData is full
*/
INT32 fpgaProgramTrdy(VOID)
{
    if(fpgaRegisterRead(FPGA_FPGA_PROGRAM_STATUS) & TRDY_BIT)
	{
        return TRUE;
    }
	else
	{
        return FALSE;
    }
}

/*******************************************************************************
* PROCEDURE
*
* NAME:         fpgaProgramRrdy
*
* SUMMARY:      return if the receive is ready
*
* SCOPE:        private
*
* DESCRIPTION:  return if the receive is ready
*
* RESTRICTIONS:
* 
* RETURNS:      TRUE:	RxData is full
*				FALSE:	RxData is empty
*/
INT32 fpgaProgramRrdy(VOID)
{
    if(fpgaRegisterRead(FPGA_FPGA_PROGRAM_STATUS) & RRDY_BIT)
	{
        return TRUE;
    }
	else
	{
        return FALSE;
    }
}

/*******************************************************************************
* PROCEDURE
*
* NAME:         fpgaProgramSsoSet
*
* SUMMARY:      Set chip selection
*
* SCOPE:        private
*
* DESCRIPTION:  Set chip selection
*
* RESTRICTIONS:
* 
* RETURNS:      
*/
VOID fpgaProgramSsoSet
 (
 BOOLEAN status
 )
{
    INT32 result;
	UINT8 temp;
	
	temp = fpgaRegisterRead(FPGA_FPGA_PROGRAM_CONTROL);
    if(status)
	{	//select device
        result = fpgaRegisterWrite(FPGA_FPGA_PROGRAM_CONTROL, temp | SSO_BIT);
    }
	else
	{
        result = fpgaRegisterWrite(FPGA_FPGA_PROGRAM_CONTROL, temp & (~SSO_BIT));
    }
}

/**************************************************/
/**  Program Interface Shift Operation Function  **/
/**************************************************/

/*******************************************************************************
* PROCEDURE
*
* NAME:         fpgaProgramShiftOut
*
* SUMMARY:      data shift out function
*
* SCOPE:        private
*
* DESCRIPTION:  wait program INT32erface is ready and shift out the data
*
* RESTRICTIONS:
* 
* RETURNS:      
*/
VOID fpgaProgramShiftOut(UINT8 data)
{
    while(!fpgaProgramTrdy());
    fpgaRegisterWrite(FPGA_FPGA_PROGRAM_TXDATA, data);
}

/*******************************************************************************
* PROCEDURE
*
* NAME:         fpgaProgramShiftOut
*
* SUMMARY:      data shift in function
*
* SCOPE:        private
*
* DESCRIPTION:  wait program INT32erface is ready shift in the data
*
* RESTRICTIONS:
* 
* RETURNS:   	byte data from EPCS   
*/
UINT8 fpgaProgramShiftIn(VOID)
{
    while(!fpgaProgramRrdy());
    return fpgaRegisterRead(FPGA_FPGA_PROGRAM_RXDATA);
	
	return -1;
}

/**********************************/
/**  Program Operation Function  **/
/**********************************/

/*******************************************************************************
* PROCEDURE
*
* NAME:         fpgaProgramWriteEnable
*
* SUMMARY:      Enable/disable EPCS write
*
* SCOPE:        private
*
* DESCRIPTION:  TRUE : 	set write enable
*	            FALSE : clear write enable
*
* RESTRICTIONS:
* 
* RETURNS:   	
*/
VOID fpgaProgramWriteEnable
 (
 BOOLEAN status
 )
{
    UINT8 temp;
    
	//Enable chip selection
    fpgaProgramSsoSet(TRUE);
	
	//Shift operation code
    if(status)
	{
        fpgaProgramShiftOut(WRITE_ENABLE); 
    }
	else
	{
        fpgaProgramShiftOut(WRITE_DISABLE);
    }
    temp = fpgaProgramShiftIn(); //throw out the garbage
    
	//Disable chip selection
    fpgaProgramSsoSet(FALSE);
    udelay(5);
}

/*******************************************************************************
* PROCEDURE
*
* NAME:         fpgaProgramReadStatus
*
* SUMMARY:      Read status byte from EPCS
*
* SCOPE:        private
*
* DESCRIPTION:  Read status byte from EPCS
*
* RESTRICTIONS:
* 
* RETURNS:   	EPCS status byte
*/
UINT8 fpgaProgramReadStatus(VOID)
{
    UINT8 returnByte, temp;
    
	//enable chip selection
    fpgaProgramSsoSet(TRUE);
    
	//shift out operation code
    fpgaProgramShiftOut(READ_STATUS);
    temp = fpgaProgramShiftIn();
    
	//shift in data
    fpgaProgramShiftOut(0);
    returnByte = fpgaProgramShiftIn();

	//disable chip selection
    fpgaProgramSsoSet(FALSE);
    
    return returnByte;
}

VOID epcsReadStatus(VOID)
{
    printk("Status: 0x%02X \n", fpgaProgramReadStatus());
}

/*******************************************************************************
* PROCEDURE
*
* NAME:         fpgaProgramReadByte
*
* SUMMARY:      Read data byte from EPCS
*
* SCOPE:        private
*
* DESCRIPTION:  Read data byte from EPCS
*
* RESTRICTIONS:
* 
* RETURNS:   	EPCS data byte
*/
UINT8 fpgaProgramReadByte
 (
 INT32 address
 )
{
    UINT8 returnByte, temp;
    
	//enable chip selection
    fpgaProgramSsoSet(TRUE);
    
	//shift out operation code
    fpgaProgramShiftOut(READ_UINT8S);
    temp = fpgaProgramShiftIn(); 
    
	//shift out highest address
    fpgaProgramShiftOut((UINT8)((address >> 16) & 0x000000FF));
    temp = fpgaProgramShiftIn(); 
    
	//shift out second highest address
    fpgaProgramShiftOut((UINT8)((address >> 8) & 0x000000FF));
    temp = fpgaProgramShiftIn(); 
    
	//shift out lowest address
    fpgaProgramShiftOut((UINT8)(address & 0x000000FF));
    temp = fpgaProgramShiftIn(); 
    
	//shift in data byte
    fpgaProgramShiftOut(0);
    returnByte = fpgaProgramShiftIn();
    
	//disable chip selection
    fpgaProgramSsoSet(FALSE);
    
    //printk("fpgaProgramReadByte: 0x%02X \n", returnByte);
    return returnByte;
}

/*******************************************************************************
* PROCEDURE
*
* NAME:         fpgaProgramReadSiliconId
*
* SUMMARY:      Read silicon ID from EPCS
*
* SCOPE:        private
*
* DESCRIPTION:  Read silicon ID from EPCS
*
* RESTRICTIONS:
* 
* RETURNS:   	EPCS silicon ID
*/
UINT8 fpgaProgramReadSiliconId(VOID)
{
    UINT8 returnByte, temp;
    
	//enable chip selection
    fpgaProgramSsoSet(TRUE);
	
	//shift out operation code
    fpgaProgramShiftOut(READ_SILICON_ID);
    temp = fpgaProgramShiftIn(); // throw out the garbage
    
	//shift out first DUMMY byte
    fpgaProgramShiftOut(DUMMY);
    temp = fpgaProgramShiftIn();
    
	//shift out second DUMMY byte
    fpgaProgramShiftOut(DUMMY);  
    temp = fpgaProgramShiftIn();
    
	//shift out third DUMMY byte
    fpgaProgramShiftOut(DUMMY);  
    temp = fpgaProgramShiftIn();
    
	//shift in silicon ID
    fpgaProgramShiftOut(0);
    returnByte = fpgaProgramShiftIn();
    
	//disable chip selection
	fpgaProgramSsoSet(FALSE);
	
    return returnByte;
}

/*******************************************************************************
* PROCEDURE
*
* NAME:         fpgaProgramWriteStatus
*
* SUMMARY:      Modify EPCS status register
*
* SCOPE:        private
*
* DESCRIPTION:  Modify EPCS status register
*
* RESTRICTIONS:
* 
* RETURNS:   	
*/

INT32 fpgaProgramWriteStatus
 (
 UINT8 data
 )
{
    UINT8 temp;
    UINT8 count = 0;
    
    //verify device presence
    if(fpgaProgramEpcsPresent())
	{
		//enable EPCS write
        fpgaProgramWriteEnable(TRUE);

		//enable chip selection
        fpgaProgramSsoSet(TRUE);
	
		//shift out operation code
        fpgaProgramShiftOut(WRITE_STATUS);
        temp = fpgaProgramShiftIn();
    
		//shift out status byte
        fpgaProgramShiftOut(data);
        temp = fpgaProgramShiftIn();
		
		//disable chip selection
        fpgaProgramSsoSet(FALSE);

        //wait until the write operation is finished
        while(fpgaProgramWriteInProgress())
		{
            mdelay(5);  //delay 5ms
            count++;
            
            if(count>100)
			{  //EPCS dead
                gEpcsDead = 1;
				printk("[FPGA EPCS] EPCS write not response!! \n");
                break;
            }
        }

        //verify that the write was successful
        if(fpgaProgramReadStatus() == data)
		{
            return TRUE;
        }
    }
		
	//EPCS is not presented
    return FALSE;
}

/*******************************************************************************
* PROCEDURE
*
* NAME:         fpgaProgramProtectLockSet
*
* SUMMARY:      set EPCS protection lock range
*
* SCOPE:        private
*
* DESCRIPTION:  set EPCS protection lock range
*
* RESTRICTIONS:
* 
* RETURNS:   	
*/
INT32 fpgaProgramProtectLockSet
 (
 UINT8 range
 )
{
	UINT8 temp;
    
	//verify device presence		
	if(fpgaProgramEpcsPresent())
	{
		//reaf the current protection range
		temp = fpgaProgramReadStatus() & 0x03;
		temp |= ((range & 0x07) << 2);
		
		//write new protection range back
		if(fpgaProgramWriteStatus(temp))
		{
			return TRUE;
		}
	}
	return FALSE;
}

/*******************************************************************************
* PROCEDURE
*
* NAME:         fpgaProgramProtectUnlock
*
* SUMMARY:      set EPCS protection lock range as 0
*
* SCOPE:        private
*
* DESCRIPTION:  set EPCS protection lock range as 0
*
* RESTRICTIONS:
* 
* RETURNS:   	
*/
INT32 fpgaProgramProtectUnlock(VOID)
{
	if(fpgaProgramProtectLockSet(0))
	{
		return TRUE;
	}
	return FALSE;
}

/*******************************************************************************
* PROCEDURE
*
* NAME:         fpgaProgramWriteByte
*
* SUMMARY:      Modify EPCS byte
*
* SCOPE:        private
*
* DESCRIPTION:  Modify EPCS byte
*
* RESTRICTIONS:
* 
* RETURNS:   	
*/
INT32 fpgaProgramWriteByte
 (
 UINT32 address, 
 UINT8 data
 )
{
    UINT8 temp;
    UINT8 count = 0;
    
    //verify device presence
    if(fpgaProgramEpcsPresent())
	{
		//unlock EPCS write protection
        fpgaProgramProtectUnlock();
		
		//enable EPCS write
        fpgaProgramWriteEnable(TRUE);
		udelay(15);
		
		//enable chip selection
        fpgaProgramSsoSet(TRUE);
		
		//shift out operation code
        fpgaProgramShiftOut(WRITE_UINT8S);
        temp = fpgaProgramShiftIn();
	
		//shift out the highest address
        fpgaProgramShiftOut((UINT8)((address >> 16) & 0x000000FF));
    	temp = fpgaProgramShiftIn(); 
    
		//shift out the second highest address
    	fpgaProgramShiftOut((UINT8)((address >> 8) & 0x000000FF));
    	temp = fpgaProgramShiftIn(); 
    
		//shift out the lowest address
    	fpgaProgramShiftOut((UINT8)(address & 0x000000FF));
    	temp = fpgaProgramShiftIn(); 
    
		//shift out data
        fpgaProgramShiftOut(data);
        temp = fpgaProgramShiftIn();    
    
		//disable chip selection
        fpgaProgramSsoSet(FALSE);
        mdelay(15);
	
		//wait until the write operation is finished
        while(fpgaProgramWriteInProgress())
		{
            mdelay(5);  //delay 5ms
            count++;
            
            if(count>100)
			{  //EPCS dead
                gEpcsDead = 1;
				printk("[FPGA EPCS] EPCS write not response!! \n");
                break;
            }
        }
        
        //verify that the write was successful    
        if(fpgaProgramReadByte(address) == data)
		{
			printk("[FPGA EPCS] Data Write Correct!! \n");
            return TRUE;
        }
		else
		{
			printk("[FPGA EPCS] Data Write Wrong!! \n");
            return FALSE;
        }
    }
	else
	{
		//EPCS is not presented
        return FALSE;
    }
}

/*******************************************************************************
* PROCEDURE
*
* NAME:         fpgaProgramEraseBulk
*
* SUMMARY:      Erase entire EPCS
*
* SCOPE:        private
*
* DESCRIPTION:  Erase entire EPCS
*
* RESTRICTIONS:
* 
* RETURNS:   	
*/
VOID fpgaProgramEraseBulk(VOID)
{
    UINT8 	temp, j = 0;
    UINT32 	count = 0;
    
    //verify device presence
    if(fpgaProgramEpcsPresent())
	{
		//unlock EPCS protection
    	fpgaProgramProtectUnlock();
    
		//enable EPCS write
    	fpgaProgramWriteEnable(TRUE);
    
		//enable chip selection
    	fpgaProgramSsoSet(TRUE);
		
		//shift out poeration code
    	fpgaProgramShiftOut(ERASE_BULK);
    	temp = fpgaProgramShiftIn();
    
		//disable chip selection
    	fpgaProgramSsoSet(FALSE);
    
		while(fpgaProgramWriteInProgress())
		{
            mdelay(5);  //delay 5ms
            count++;
            j++;
	    
			if(j == 100)
			{
				//printk("[FPGA EPCS] Waiting time %d ms\n", count*5);
				j = 0;
			}
			    
            if(count>10000)
			{  //EPCS dead
                gEpcsDead = 1;
				printk("[FPGA EPCS] EPCS write not response!! \n");
                break;
            }
        }

        //verify that the write was successful
        if(fpgaProgramReadStatus() == 0x00)
		{
            printk("[FPGA EPCS] EPCS erase is done! \n");
        }
	}
	else
	{     
    	printk("[FPGA EPCS] EPCS erase is failed! \n");   
    }
}

/*******************************************************************************
* PROCEDURE
*
* NAME:         fpgaProgramEraseSector
*
* SUMMARY:      Erase EPCS sector
*
* SCOPE:        private
*
* DESCRIPTION:  Erase EPCS sector
*
* RESTRICTIONS:
* 
* RETURNS:   	
*/
VOID fpgaProgramEraseSector
 (
 UINT32 address	//sector start address
 )
{
    UINT8 	temp, j = 0;
    UINT32 count = 0;
    
	//printk("[FPGA EPCS] Erasing address: 0x%08X\n", address);
	
    //verify device presence
    if(fpgaProgramEpcsPresent())
	{
		//unlock EPCS protection
    	fpgaProgramProtectUnlock();
    
		//enable EPCS write
    	fpgaProgramWriteEnable(TRUE);
    
		//enable chip selection
    	fpgaProgramSsoSet(TRUE);
    
		//shift out operation code
    	fpgaProgramShiftOut(ERASE_SECTOR);
    	temp = fpgaProgramShiftIn();
    
		//shift out the highest address
    	fpgaProgramShiftOut((UINT8)((address >> 16) & 0x000000FF));
    	temp = fpgaProgramShiftIn(); 
    
		//shift our the second highest address
    	fpgaProgramShiftOut((UINT8)((address >> 8) & 0x000000FF));
    	temp = fpgaProgramShiftIn(); 
    
		//shit out the lowest address
    	fpgaProgramShiftOut((UINT8)(address & 0x000000FF));
    	temp = fpgaProgramShiftIn(); 

		//disable chip selection
    	fpgaProgramSsoSet(FALSE);
		
    	while(fpgaProgramWriteInProgress())
		{
            mdelay(5);  //delay 5ms
            count++;
            j++;
	    
			if(j == 100)
			{
				//printk("[FPGA EPCS] Waiting time %d ms\n", count*5);
				j = 0;
			}
			    
            if(count>10000)
			{  //EPCS dead
                gEpcsDead = 1;
				printk("[FPGA EPCS] EPCS write not response!! \n");
                break;
            }
        }

        //verify that the write was successful
        if(fpgaProgramReadStatus() == 0x00)
		{
            //printk("[FPGA EPCS] EPCS erase sector is done! \n");
        }
    }
	else
	{     
    	printk("[FPGA EPCS] EPCS erase sector is failed! \n");   
    }
}

/*******************************************************************************
* PROCEDURE
*
* NAME:         fpgaProgramReadBuffer
*
* SUMMARY:      Read EPCS content
*
* SCOPE:        private
*
* DESCRIPTION:  Read EPCS content
*
* RESTRICTIONS:
* 
* RETURNS:   	
*/
INT32 fpgaProgramReadBuffer
 (
 UINT32 	address, 
 INT32 		length, 
 UINT8 		*data
 )
{
    UINT8 temp;
    
    //verify device presence
    if(fpgaProgramEpcsPresent())
	{
		//enable chip selection
        fpgaProgramSsoSet(TRUE);
		
		//shift out operation code
        fpgaProgramShiftOut(READ_UINT8S);
        temp = fpgaRegisterRead(FPGA_FPGA_PROGRAM_RXDATA); 
        
		//shift out the highest address
    	fpgaProgramShiftOut((UINT8)((address >> 16) & 0x000000FF));
    	temp = fpgaProgramShiftIn(); 
    
		//shift out the second highest address
    	fpgaProgramShiftOut((UINT8)((address >> 8) & 0x000000FF));
    	temp = fpgaProgramShiftIn(); 
    
		//shift out the lowest address
    	fpgaProgramShiftOut((UINT8)(address & 0x000000FF));
    	temp = fpgaProgramShiftIn(); 
    
		//shift in EPCS content
        while (length-- > 0)
		{
            fpgaProgramShiftOut(0);
            *data++ = fpgaProgramShiftIn();
        }

		//disable chip selection
        fpgaProgramSsoSet(FALSE);
        return TRUE;
    }
	//EPCS is not presented
    return FALSE;
}

VOID epcsReadBuffer
 (
 UINT32 address
 )
{
    UINT8  	test[256];
    UINT32 	i;
    
	//prINT32 256 bytes content
    fpgaProgramReadBuffer(address, 256, test);
     
    for(i = 0; i < 256; i++)
	{
    	printk("Address: 0x%08X, data: 0x%02X \n", i, test[i]); 
    }
}

/*******************************************************************************
* PROCEDURE
*
* NAME:         fpgaProgramWritePage
*
* SUMMARY:      Write EPCS content
*
* SCOPE:        private
*
* DESCRIPTION:  Write EPCS content. the max. size is 256 bytes.
*
* RESTRICTIONS:
* 
* RETURNS:   	
*/
INT32 fpgaProgramWritePage
 (
 UINT32 	address, 
 INT32 		length, 
 UINT8 		*data
 )
{
    UINT8 	verify[length];
    INT32 	tempLength;
    UINT8 	*tempData;
    INT32 	count = 0;
    INT32 	i;
    UINT8 	temp, s, d;
    
    tempData = data;

    if(length > 256)
	{
    	printk("[FPGA EPCS] Data length is larger than 256 bytes!! Write abort.\n");
		return FALSE;
    }
    
    //inverse byte
    for(tempLength = length; tempLength > 0; tempLength--, tempData++)
	{
        s = *tempData;
        if((s == 0) || (s == 0xFF))
            continue;
        d = 0;
        for(i=0; i<8; i++)
		{
            d <<= 1;
            if(s & 1)
                d |= 1;
            s >>= 1;
        }
        *tempData = d;
    }
    
    tempData = data;
    tempLength = length;
        
    //verify device presence
    if(fpgaProgramEpcsPresent())
	{
		//unlock EPCS protection
		fpgaProgramProtectUnlock();
	
		//enable EPCS write
        fpgaProgramWriteEnable(TRUE);
	
		//enable chip selection
        fpgaProgramSsoSet(TRUE);
        fpgaProgramShiftOut(WRITE_UINT8S);
        temp = fpgaRegisterRead(FPGA_FPGA_PROGRAM_RXDATA); 
    
		//shift out address
    	fpgaProgramShiftOut((UINT8)((address >> 16) & 0x000000FF));
    	temp = fpgaProgramShiftIn(); 
    
    	fpgaProgramShiftOut((UINT8)((address >> 8) & 0x000000FF));
    	temp = fpgaProgramShiftIn(); 
    
    	fpgaProgramShiftOut((UINT8)(address & 0x000000FF));
    	temp = fpgaProgramShiftIn(); 
    
        //shift out the data
        while (tempLength-- > 0)
		{  
            fpgaProgramShiftOut(*tempData);
            temp = fpgaRegisterRead(FPGA_FPGA_PROGRAM_RXDATA); 
            tempData++;
        }

		//disable chip selection
        fpgaProgramSsoSet(FALSE);
	
        //wait until the write operation is finished
        while(fpgaProgramWriteInProgress())
		{
            mdelay(5);  //delay 5ms
            count++;
            
            if(count>1200)
			{  //EPCS dead
                gEpcsDead = 1;
                return FALSE;   
            }
        }
	
        //verify that the data was correctly written
        fpgaProgramReadBuffer(address, length, verify);
        for(count = 0; count < length; count++)
		{
            if (verify[count] != data[count])
			{
				printk("[FPGA EPCS] Page write failed: 0x%08X, expect: 0x%02X, return: 0x%02X \n", count, data[count], verify[count]);
                return FALSE;
            }
        }
	
        return TRUE;
    }
    //EPCS is not presented
    return FALSE;
   
}

/*******************************************************************************
* PROCEDURE
*
* NAME:         fpgaProgramWriteBuffer
*
* SUMMARY:      Write EPCS content
*
* SCOPE:        private
*
* DESCRIPTION:  Write EPCS content.
*
* RESTRICTIONS:
* 
* RETURNS:   	
*/
INT32 fpgaProgramWriteBuffer
 (
 UINT32 	address, 
 INT32 		length, 
 UINT8 		*data
 )
{
    INT32 firstPage;
    INT32 totalLength;
    
    totalLength = length;
    //verify device presence
    if(fpgaProgramEpcsPresent())
	{
		//unlock EPCS protection
		fpgaProgramProtectUnlock();
		
        //shift out the first page (if length > one page)
        firstPage = address % EPCS_PAGE_SIZE;
        if ((firstPage != 0) && ((firstPage + length) > EPCS_PAGE_SIZE))
        {
            if(!fpgaProgramWritePage(address, (EPCS_PAGE_SIZE - firstPage), data))
			{
                return FALSE;       //write failed
            }
            
            address += firstPage;
            data += firstPage;
            length -= firstPage;
	    
			commPostProgress("[FPGA EPCS] Write Progress", firstPage, totalLength);
        }

        //shift out other pages
        while ((length / EPCS_PAGE_SIZE) > 0)
		{
            if(!fpgaProgramWritePage(address, EPCS_PAGE_SIZE, data))
			{
                return FALSE;       //write failed
            }
            
            address += EPCS_PAGE_SIZE;
            data += EPCS_PAGE_SIZE;
            length -= EPCS_PAGE_SIZE;
			firstPage += EPCS_PAGE_SIZE;
	    
			commPostProgress("[FPGA EPCS] Write Progress", firstPage, totalLength);
        }

        //shift out the last page
        if (length > 0)
        {
            if(!fpgaProgramWritePage(address, EPCS_PAGE_SIZE, data))
			{
                return FALSE;       //write failed
            }
        }

        return TRUE;
    }
	else
	{	//EPCS is not presented
        return FALSE;
    }
}


/*****************************************************/
/**  Program Interface <EPCS Side> Status Function  **/
/*****************************************************/

/*******************************************************************************
* PROCEDURE
*
* NAME:         fpgaProgramWriteComplete
*
* SUMMARY:      verify EPCS writing is complete
*
* SCOPE:        private
*
* DESCRIPTION:  verify EPCS writing is complete
*
* RESTRICTIONS:
* 
* RETURNS:   	TRUE: 	EPCS is in progress
*				FALSE: 	EPCS is not progress
*/
INT32 fpgaProgramWriteComplete(VOID)
{
    UINT32 timeOut = 11770000;
    UINT32 i = 0;
	
    while((!fpgaProgramReadStatus()) && (i < timeOut))
	{
    	i++;
    }

    if(i < timeOut)
	{
        return TRUE;
    }
	else
	{
        return FALSE;
    }
}

/*******************************************************************************
* PROCEDURE
*
* NAME:         fpgaProgramWriteInProgress
*
* SUMMARY:      verify EPCS writing is in process
*
* SCOPE:        private
*
* DESCRIPTION:  verify EPCS writing is in process
*
* RESTRICTIONS:
* 
* RETURNS:   	TRUE: 	EPCS writing is in progress
*				FALSE: 	EPCS writing is complete
*/
INT32 fpgaProgramWriteInProgress(VOID)
{
    if(fpgaProgramReadStatus() & 0x01)
	{
        return TRUE;
    }
	else
	{
        return FALSE;
    }
}

/*******************************************************************************
* PROCEDURE
*
* NAME:         fpgaProgramEpcsPresent
*
* SUMMARY:      verify EPCS is present
*
* SCOPE:        private
*
* DESCRIPTION:  verify EPCS is present
*
* RESTRICTIONS:
* 
* RETURNS:   	TRUE: 	EPCS is present
*				FALSE: 	EPCS is not present
*/
INT32 fpgaProgramEpcsPresent(VOID)
{
	//enable EPCS write
    fpgaProgramWriteEnable(TRUE);

    if(fpgaProgramReadStatus() & 0x02)
    {
        fpgaProgramWriteEnable(FALSE);

        if(!(fpgaProgramReadStatus() & 0x02))
        {    
			gEpcsDead = 0;
            return TRUE;        //EPCS exists
		}
    }
    
    gEpcsDead = 1;
    return FALSE;
}

/*******************************************************************************
* PROCEDURE
*
* NAME:         fpgaProgramEpcs
*
* SUMMARY:      program EPCS
*
* SCOPE:        private
*
* DESCRIPTION:  Program EPCS
*
* RESTRICTIONS:
* 
* RETURNS:   	
*/
INT32 fpgaProgramEpcs
 (
 UINT32 	address, 
 INT32 		length, 
 UINT8 		*data
 )
{
	INT32	i;
	
	//verify the over part
    if(address > (EPCS_BULK_SIZE - 1))
	{
		printk("[FPGA EPCS] EPCS4 Write Over. \n");
		return 0;
	}

	//erase
	for(i = 0; i < EPCS_SECTOR_NUM; i++)
	{
		fpgaProgramEraseSector(address + (i * EPCS_SECTOR_SIZE));
		commPostProgress("[FPGA EPCS] Erase Progress", i+1, EPCS_SECTOR_NUM);
	}
	
	//reprogram			
	if(!fpgaProgramWriteBuffer(address, length, data))
	{
		return -1;
	}

	return 0;
}
EXPORT_SYMBOL(fpgaProgramEpcs);


/*******************************************************************************
* PROCEDURE
*
* NAME:         fpgaProgramInterfaceDiag
*
* SUMMARY:      diagnostics EPCS programming INT32erface
*
* SCOPE:        private
*
* DESCRIPTION:  diagnostics EPCS programming INT32erface
*
* RESTRICTIONS:
* 
* RETURNS:   	TRUE:	Pass
*				FALSE:	Fail
*/
INT32 fpgaProgramInterfaceDiag(VOID)
{
    if(!fpgaProgramEpcsPresent())
	{   //verify EPCS is present
        commReportError(MEC_FPGA, FPGA_PROGRAM_INTERFACE_MEC, FPGA_EPCS_ERROR,
						MSG_FPGA_EPCS_ERROR, 0, 0, 0, 0);	
        return FALSE;
    }
    
    if(fpgaProgramReadSiliconId() != EPCS4_ID)
	{    //if return ID is not EPCS1 or EPCS4
        commReportError(MEC_FPGA, FPGA_PROGRAM_INTERFACE_MEC, FPGA_PROGRAM_INTERFACE_ERROR,
						MSG_FPGA_PROGRAM_INTERFACE_ERROR, 0, 0, 0, 0);
		printk("[FPGA Program Interface] Silicon ID Read: 0x%02X \n", fpgaProgramReadSiliconId());	
        return FALSE;
    }
	else
	{	        
        return TRUE;
    }
}

