/// @file       NVMeDDAPI.cpp
/// @brief      NVMeDDAPI class
/// @details    Provide low layer diagnostic and utility APIs for NVMe module
/// @version    1.0.0
/// @date       2014.02.21
/// @author     Jerry Lee
/// @copyright  Copyright 2014 Hon Hai Technology Group. All Rights Reserved.
#include <string.h>
#include "NVMeDDAPI.hpp"

#define min(x, y) (x) > (y) ? (y) : (x)
static unsigned char nvmeControllerID[] = {
    0, 1
};

/// @fn         NVMeDDAPI::NVMeDDAPI(STRING className) : BaseClassDDAPI(className)
/// @brief      NVMeDDAPI constructor
/// @param      in_szClassName is a char string for class name
/// @exception  Nothing
/// @return     Nothing
NVMeDDAPI::NVMeDDAPI(STRING className) : BaseClassDDAPI(className)
{
    ErrorMsg* m_errorMsg = NO_ERROR_MSG;
    nvmeIOCTL = NULL;

    // initialize here
    if((pciDDAPI = new PCIDDAPI("PCI_DRVR")) == NULL)
    {
        addErrorMsg((ERRORCODE)(ERROR_NVME_DDAPI_ERRORCODE | ERROR_DIAGNOSTIC_DRIVER_API_CONSTRUCTOR1_ERROR), "Create PCI DDAPI fail");
    }

    addNVMeController(); // search and add IOC information on the host

    if(nvmeList.empty())
        addErrorMsg((ERRORCODE)(ERROR_DIAGNOSTIC_DRIVER_API_CONSTRUCTOR1_ERROR | ERROR_NVME_DDAPI_NO_NVME_CONTROLLER), NULL, NULL);

    for(vector<NVMe_DEVICE_INFO>::iterator iter = nvmeList.begin(); iter != nvmeList.end(); iter++)
    {
        m_errorMsg = getDeviceNodeByController(iter - nvmeList.begin());
        if(m_errorMsg == NO_ERROR_MSG)
            m_errorMsg = doIOCTLCommand((iter - nvmeList.begin()), NVMeIOCTL::NAMESPACE_INFORMATION, &iter->idNs);
        if(m_errorMsg)
            addErrorMsg (ERRORCODE(ERROR_DIAGNOSTIC_DRIVER_API_CONSTRUCTOR1_ERROR | ERROR_NVME_DDAPI_GET_NVME_LIST), NULL, m_errorMsg);
    }

}

/// @fn         NVMeDDAPI::~NVMeDDAPI(void)
/// @brief      NVMeDDAPI destructor
/// @exception  Nothing
/// @return     Nothing
NVMeDDAPI::~NVMeDDAPI(void)
{
    // delete PCI Driver API
    if (pciDDAPI != NULL)
        delete pciDDAPI;

    // delete IOCTL object we created
    if (nvmeIOCTL != NULL)
        delete nvmeIOCTL;
}

/// @fn         NVMeDAPI::addNVMeController(void)
/// @brief      add all IOCs' information on the host
/// @param      None
/// @exception  Nothing
/// @return     A pointer to ErrorMsg for error message list
ErrorMsg*
NVMeDDAPI::addNVMeController(void)
{
    NVMe_DEVICE_INFO nvmeInfo;
    vector<PCIInformantion> pciInfo;
    ErrorMsg* errorMsg = NULL;

    errorMsg = pciDDAPI->getPciInfoByClass(PCI_NVME, &pciInfo);
    if(errorMsg)
        return returnErrorMsg(ERRORCODE(ERROR_NVME_DDAPI_CONSTRUCTOR), NULL, errorMsg);

    for (vector<PCIInformantion>::iterator iter = pciInfo.begin(); iter != pciInfo.end(); iter++)
    {
        if (errorMsg == (ErrorMsg *) NO_ERROR)
        {
            nvmeInfo.wellKnowName = "NVME" + Int2Str(nvmeControllerID[iter - pciInfo.begin()]);
            nvmeInfo.domindBusDeviceFunction = Int2HexStr(iter->u16Domain,4) + ":"
                    + Int2HexStr(iter->u16Bus) + ":" + Int2HexStr(iter->u16Device) + "."
                    + Int2HexStr(iter->u8Function, 1);

            nvmeList.push_back(nvmeInfo);
        }
    }

    std::sort(nvmeList.begin(), nvmeList.end(), sortNVMEcontroller);

    if(errorMsg)
        return returnErrorMsg(ERRORCODE(ERROR_NVME_DDAPI_CONSTRUCTOR), NULL, errorMsg);

    return NO_ERROR_MSG;
}

/// @fn         NVMeDDAPI::doIOCTLCommand
/// @brief      execute NVMe controller IOCTL command
/// @param      NVMe logical ID
/// @param      NVMe_IOCTL_REQUEST
/// @param      The returned value
/// @exception  Nothing
/// @return     A pointer to ErrorMsg for error message list
ErrorMsg*
NVMeDDAPI::doIOCTLCommand(UINT8 controllerID, NVMeIOCTL::IOCTL_OP_CODE ioctlCommand, void* returnedValue)
{
    ErrorMsg* errorMsg = NULL;

    if(nvmeList.empty())
    {
        return returnErrorMsg((ERRORCODE)(ERROR_NVME_DDAPI_EXECUTE_IOCTL | ERROR_NVME_DDAPI_NO_NVME_CONTROLLER), NULL, NULL);
    }

    for(vector<NVMe_DEVICE_INFO>::iterator iter = nvmeList.begin(); iter != nvmeList.end(); iter++)
    {
        if (controllerID == (iter - nvmeList.begin()))
        {
            if((nvmeIOCTL = new NVMeIOCTL((iter->blockdeviceNodeName).c_str())) == NULL)
            {
                returnErrorMsg((ERRORCODE)(ERROR_NVME_DDAPI_EXECUTE_IOCTL | ERROR_NVME_DDAPI_OPEN_NVME_DEVICE_FAIL), "Create NVMe IOCTL fail");
            }

            if((m_errorCode = nvmeIOCTL->allocateIOCTLArgument(ioctlCommand)) == NO_ERROR)
            {
                if ((m_errorCode = nvmeIOCTL->setIOCTLOperationArgument(ioctlCommand, returnedValue)) == NO_ERROR)
                // separate ioctl command here to distinguish the command has return data or customized argument
                switch(ioctlCommand)
                {
                    case NVMeIOCTL::EXECUTE_NVME_RESET:
                    case NVMeIOCTL::EXECUTE_NVME_SUBSYSTEM_RESET:
                        nvmeIOCTL->setFileHandle((iter->chardeviceNodeName).c_str());
                        m_errorCode = nvmeIOCTL->executeIOCTL(ioctlCommand);

                        break;
                    case NVMeIOCTL::CONTROLLER_INFORMATION:
                    case NVMeIOCTL::NAMESPACE_INFORMATION:
                    case NVMeIOCTL::FIRMWARE_LOG:
                    case NVMeIOCTL::SMART_LOG:
                    case NVMeIOCTL::ERROR_LOG:
                    case NVMeIOCTL::FIRMWARE_ACTIVATE:
                        m_errorCode = nvmeIOCTL->executeIOCTL(ioctlCommand);

                        break;
                    case NVMeIOCTL::FORMAT:
                    case NVMeIOCTL::WRITE_ZERO:
                        if((m_errorCode = nvmeIOCTL->setIOCTLOperationArgument(ioctlCommand, returnedValue)) == NO_ERROR)
                            m_errorCode = nvmeIOCTL->executeIOCTL(ioctlCommand);

                        break;

                    case NVMeIOCTL::NAMESPACE_ID:
                        m_errorCode = nvmeIOCTL->executeIOCTL(ioctlCommand, returnedValue);

                        break;
                    case NVMeIOCTL::UnspportIOCTLCommand:
                        m_errorCode = ERROR_IOCTL_INVALID_COMMAND_CODE;

                        break;
                    case NVMeIOCTL::FIRMWARE_DOWNLOAD:
                        break;
                }
            }
            break;
        }
    }

    if(m_errorCode != NO_ERROR)
        return returnErrorMsg(ERRORCODE(ERROR_NVME_DDAPI_EXECUTE_IOCTL | m_errorCode), NULL, NULL);

    return NO_ERROR_MSG;
}

/// @fn         NVMeDDAPI::getControllerList
/// @brief      Get NVMe information list on the host
/// @param      NVMe logical ID
/// @param      A pointer to NVMe controller list
/// @exception  Nothing
/// @return     A pointer to ErrorMsg for error message list
ErrorMsg*
NVMeDDAPI::getControllerList(vector<NVMe_DEVICE_INFO>* returnedList)
{
    if(nvmeList.empty())
        return returnErrorMsg((ERRORCODE)(ERROR_NVME_DDAPI_GET_NVME_LIST | ERROR_NVME_DDAPI_NO_NVME_CONTROLLER), NULL, NULL);

    *returnedList = nvmeList;

    return returnErrorMsg(NO_ERROR, NULL, NULL);
}

/// @fn         NVMeDDAPI::getDeviceNodeByController(UINT8 controllerID)
/// @brief      get specified NVME device node name
/// @param      NVMe logical ID
/// @exception  Nothing
/// @return     A pointer to ErrorMsg for error message list
ErrorMsg*
NVMeDDAPI::getDeviceNodeByController(UINT8 controllerID)
{
    string pciBDF;
    CHAR* charDirToRead = (CHAR *)"/sys/class/nvme/";
    CHAR* blockDirToRead = (CHAR *)"/sys/block/";
    DIR* chardir = opendir(charDirToRead);
    DIR* blockdir = opendir(blockDirToRead);
    struct dirent *dirent;
    struct stat fileAttribute;
    CHAR tempBuffer[256];
    CHAR fileName[256] = "";

    if(nvmeList.empty())
    {
        return returnErrorMsg((ERRORCODE)(ERROR_NVME_DDAPI_GET_DEVICE_NODE_LIST | ERROR_NVME_DDAPI_NO_NVME_CONTROLLER), NULL, NULL);
    }
    if(blockdir == NULL || chardir == NULL)
    {
        return returnErrorMsg(ERRORCODE(ERROR_NVME_DDAPI_GET_DEVICE_NODE_LIST | ERROR_OPEN_DIRECTORY), NULL, NULL);
    }
    for(vector<NVMe_DEVICE_INFO>::iterator iter = nvmeList.begin(); iter != nvmeList.end(); iter++)
    {
        if (controllerID == (iter - nvmeList.begin()))
        {
            pciBDF = iter->domindBusDeviceFunction;
            while((dirent = readdir(chardir)) != NULL)
            {
                snprintf(fileName, sizeof(fileName), "%s%s", charDirToRead, dirent->d_name);
                lstat(fileName, &fileAttribute);

                if(S_ISDIR(fileAttribute.st_mode))
                {
                    // This directory dirent is another directory
                }
                else if(S_ISLNK(fileAttribute.st_mode))
                {
                    INT32 linkLength = readlink(fileName, tempBuffer, sizeof(tempBuffer) - 1);
                    if(linkLength != -1)
                    {
                        tempBuffer[linkLength] = '\0';
                    }
                    else
                    {
                        closedir(chardir);
                        return returnErrorMsg(ERRORCODE(ERROR_NVME_DDAPI_GET_DEVICE_NODE_LIST | ERROR_NVME_DDAPI_READ_LINK_ERROR), NULL, NULL);;
                    }

                    CHAR tempString[256] = "";
                    snprintf(tempString, sizeof(tempString), "%s", pciBDF.c_str());
                    CHAR *chpAddress = strstr(tempBuffer, tempString);

                    if(chpAddress == NULL)
                    {
                    continue;
                    }

                    iter->chardeviceNodeName = (string)"/dev/" + dirent->d_name;
                }
                else if(S_ISREG(fileAttribute.st_mode))
                {
                  // This dirent is a regular filename
                }
            }
            closedir(chardir);

            while((dirent = readdir(blockdir)) != NULL)
            {
                snprintf(fileName, sizeof(fileName), "%s%s", blockDirToRead, dirent->d_name);
                lstat(fileName, &fileAttribute);

                if(S_ISDIR(fileAttribute.st_mode))
                {
                    // This directory dirent is another directory
                }
                else if(S_ISLNK(fileAttribute.st_mode))
                {
                    INT32 linkLength = readlink(fileName, tempBuffer, sizeof(tempBuffer) - 1);
                    if(linkLength != -1)
                    {
                        tempBuffer[linkLength] = '\0';
                    }
                    else
                    {
                        closedir(blockdir);
                        return returnErrorMsg(ERRORCODE(ERROR_NVME_DDAPI_GET_DEVICE_NODE_LIST | ERROR_NVME_DDAPI_READ_LINK_ERROR), NULL, NULL);;
                    }

                    CHAR tempString[256] = "";
                    snprintf(tempString, sizeof(tempString), "%s", pciBDF.c_str());
                    CHAR *chpAddress = strstr(tempBuffer, tempString);

                    if(chpAddress == NULL)
                    {
                    continue;
                    }

                    iter->blockdeviceNodeName = (string)"/dev/" + dirent->d_name;
                }
                else if(S_ISREG(fileAttribute.st_mode))
                {
                  // This dirent is a regular filename
                }
            }
            closedir(blockdir);
            break;
        }
    }

    return NO_ERROR_MSG;
}

/// @fn         NVMeDDAPI::downloadFirmware(UINT8 controllerID, void* buffer, UINT32 size)
/// @brief      download image to the requested NVMe device
/// @param      IOC ID
/// @param      image buffer
/// @param      image buffer size
/// @exception  Nothing
/// @return     A pointer to ErrorMsg for error message list
ErrorMsg*
NVMeDDAPI::downloadFirmware(UINT8 controllerID, void* buffer, UINT32 size)
{
    UINT32 tempSize;
    nvme_fw_download_info fwInfo;
    memset(&fwInfo, 0, sizeof(fwInfo));

    fwInfo.fw_buf = buffer;
    tempSize = size;

    if(nvmeList.empty())
    {
        return returnErrorMsg((ERRORCODE)(ERROR_NVME_DDAPI_DOWNLOAD_FW | ERROR_NVME_DDAPI_NO_NVME_CONTROLLER), NULL, NULL);
    }

    for(vector<NVMe_DEVICE_INFO>::iterator iter = nvmeList.begin(); iter != nvmeList.end(); iter++)
    {
        if (controllerID == (iter - nvmeList.begin()))
        {
            if((nvmeIOCTL = new NVMeIOCTL((iter->blockdeviceNodeName).c_str())) == NULL)
            {
                returnErrorMsg((ERRORCODE)(ERROR_NVME_DDAPI_DOWNLOAD_FW | ERROR_NVME_DDAPI_OPEN_NVME_DEVICE_FAIL), "Create NVMe IOCTL fail");
            }

            if((m_errorCode = nvmeIOCTL->allocateIOCTLArgument(NVMeIOCTL::FIRMWARE_DOWNLOAD)) == NO_ERROR)
            {
                while (tempSize > 0) {
                    fwInfo.xfer = min(4096, tempSize);
                    nvmeIOCTL->setIOCTLOperationArgument(NVMeIOCTL::FIRMWARE_DOWNLOAD, &fwInfo);
                    if((m_errorCode = nvmeIOCTL->executeIOCTL(NVMeIOCTL::FIRMWARE_DOWNLOAD))!= NO_ERROR)
                        return returnErrorMsg(ERRORCODE(ERROR_NVME_DDAPI_DOWNLOAD_FW | m_errorCode), NULL, NULL);

                    fwInfo.fw_buf   += fwInfo.xfer;
                    tempSize        -= fwInfo.xfer;
                    fwInfo.offset   += fwInfo.xfer;
                }
            }
            break;
        }
    }

    return NO_ERROR_MSG;
}

bool
NVMeDDAPI::sortNVMEcontroller(const NVMe_DEVICE_INFO &left, const NVMe_DEVICE_INFO &right)
{
    return left.wellKnowName < right.wellKnowName;
}
